/*
DEVICE DESCRIPTION FOR ABB Instrumentation (TBI) TB82EC CONCENTRATION TRANSMITTER

Copyright 2000 ABB Automation Inc., all rights reserved.
This software may not be used, either directly or by reference,
such that the software provides functionality to an end customer,
unless explicitly lisenced by ABB Automation Inc.

Author: Shanthala Narayanaswamy
        ABB Instrumentation
        29801 Euclid Avenue
        Wickliffe, Ohio USA 44092

Date:   17 August 2000
*/

#include "macros.ddl"
#include "bi_codes.h"

MANUFACTURER BAILEY_FISCHER_AND_PORTER, DEVICE_TYPE _TB82EC_CONC, DEVICE_REVISION 1, DD_REVISION 1

/*
COMMANDS SUPPORTED IN THE TB82EC-CONC TRANSMITTER:

UNIVERSAL COMMANDS:
0  Read Unique Identifier
1  Read Primary Variable
2  Read PV Current and Percent of Range
3  Read Dynamic Variables and PV Current
6  Write Polling Address
11 Read Unique Identifier Associated with Tag
12 Read Message
13 Read Tag, Descriptor, Date
14 Read Primary Variable Sensor Information
15 Read Primary Variable Output Information
16 Read Final Assembly Number
17 Write Message
18 Write Tag, Descriptor, Date
19 Write Final Assembly Number

COMMON PRACTICE COMMANDS:
34 Write Primary Variable Damping Value
35 Write Primary Variable Range Values
36 Set Primary Variable Upper Range Value
37 Set Primary Variable Lower Range Value
38 Reset Configuration Changed Flag
40 Enter/Exit Fixed Primary Variable Current Mode
42 Perform Master Reset
43 Set Primary Variable Zero
44 Write Primary Variable Units
45 Trim Primary Variable Current DAC Zero
46 Trim Primary Variable Current DAC Gain
48 Read Additional Transmitter Status
49 Write Primary Variable Sensor Serial Number
59 Write Number of Response Preambles

TRANSMITTER SPECIFIC COMMANDS:
128 Reset to factory calibration 
129 Read calibration efficiency and offset
130 Write calibration efficiency and offset
135 1 Point Capture
136 One Point Calibration 
138 Temperature Sensor Calibration 
*/


/****************************************************************************
**			IMPORT STANDARD DDs
*****************************************************************************/

/*
*       COMMON TABLES
*/
IMPORT STANDARD _TABLES, DEVICE_REVISION 9, DD_REVISION 2
{
    VARIABLE units_code;
    VARIABLE transfer_function_code;
    VARIABLE alarm_selection_code;
    VARIABLE write_protect_code;
    VARIABLE company_identification_code;
}

/*
*       UNIVERSAL COMMANDS
*/
IMPORT STANDARD _UNIVERSAL, DEVICE_REVISION 5, DD_REVISION 1
{
    EVERYTHING;
    REDEFINITIONS
    {
        VARIABLE device_type
        {
            REDEFINE TYPE ENUMERATED
            {
                {85, "82EC CONC", "Four-Electrode Concentration Transmitter|de|Vier-Elektroden Konzentrations-Meumformer"}
            }
        }
        VARIABLE manufacturer_id
        {
            REDEFINE TYPE ENUMERATED
            {
                {18, "ABB Instrumentation"}
            }
        }
        VARIABLE private_label_distributor
        {
            REDEFINE TYPE ENUMERATED
            {
                {18, "ABB Instrumentation"}
            }
        }
        VARIABLE device_status
        {
            REDEFINE TYPE BIT_ENUMERATED
            {
                { 0x01, [PV_out_of_limits],        DV1 (AUTO & BAD) },
                { 0x02, [Non_PV_out_of_limits],    DV2 (AUTO & BAD) },
                { 0x04, [PV_analog_output_saturated],  AO1 (AUTO & BAD) },
                { 0x08, [PV_analog_output_fixed],      AO1 (MANUAL & GOOD) },
                { 0x10, [more_status_available],   MORE & IGNORE_IN_HANDHELD },
                { 0x20, [cold_start],          MISC },
                { 0x40, [configuration_changed],   MISC & IGNORE_IN_HANDHELD },
                { 0x80, [device_malfunction],      STATE & SUMMARY} /* MORE is 
                **not used for this bit because, this will result in two cmd 48s 
                **to be sent out during calibration, which results in the DD missing the
                **PV cal errors that are available with only the first response
                ** to cmd 48 after a cal command. */
            }
        }
        COMMAND read_dynamic_variables_and_pv_current
        {
            REDEFINE TRANSACTION
            {
                REQUEST {}
                REPLY
                {
                    /* The TB82EC-CONC Transmitter only has two dynamic variables:
                       Concentration and temperature */

                    response_code, device_type,
                    PV.ANALOG_VALUE,
                    PV.DIGITAL_UNITS,
                    PV.DIGITAL_VALUE,
                    SV.DIGITAL_UNITS,
                    SV.DIGITAL_VALUE
                }
            }
        }
    }
}

/*
*   COMMON PRACTICE COMMANDS
*/

IMPORT STANDARD _COMMON_PRACTICE, DEVICE_REVISION 7, DD_REVISION 2
{
   /* The TB82EC-CONC Transmitter has 9 extended status bytes */

   VARIABLE xmtr_specific_status_0;
   VARIABLE xmtr_specific_status_1;
   VARIABLE xmtr_specific_status_2;
   VARIABLE xmtr_specific_status_3;
   VARIABLE xmtr_specific_status_4;
   VARIABLE xmtr_specific_status_5;
   VARIABLE xmtr_specific_status_6;
   VARIABLE xmtr_specific_status_7;
   VARIABLE xmtr_specific_status_8;
   VARIABLE response_preambles;

   COMMAND write_pv_damping_value;             /* Command 34 */
   COMMAND write_pv_range_values;              /* Command 35 */
   COMMAND set_pv_upper_range_value;           /* Command 36 */
   COMMAND set_pv_lower_range_value;           /* Command 37 */
   COMMAND reset_configuration_change_flag;    /* Command 38 */
   COMMAND enter_exit_fixed_pv_current_mode;   /* Command 40 */
   COMMAND perform_master_reset;               /* Command 42 */
   COMMAND set_pv_zero;                        /* Command 43 */
   COMMAND write_pv_units;                     /* Command 44 */
   COMMAND trim_pv_current_dac_zero;           /* Command 45 */
   COMMAND trim_pv_current_dac_gain;           /* Command 46 */
   COMMAND read_additional_transmitter_status; /* Command 48 */
   COMMAND write_pv_sensor_serial_number;      /* Command 48 */
   COMMAND write_number_of_response_preambles; /* Command 59 */

   METHOD loop_test;
   METHOD dac_trim;
   METHOD scaled_dac_trim;
   METHOD leave_fixed_current_mode;
   METHOD return_to_normal;
   METHOD warning_message;

   REDEFINITIONS
   {
      /* The following are the definitions for the 7 extended status bytes
      supplied by the transmitter.  Bytes 6 thru 13 are not used
      by the transmitter.  It should be ignored except for communications purposes. */

      VARIABLE xmtr_specific_status_0
      {
         REDEFINE TYPE BIT_ENUMERATED
         {
            { 0x01, "EC1 HI.PV.AD",
                    "Process Variable A/D over range"
                    "|de|berschreitung des A/D Wandlerbereichs (Prozevariable)", 
                    HARDWARE & UNCORRECTABLE},

            { 0x02, "EC2 LO.PV.AD", 
                    "Process Variable A/D under range"
                    "|de|Unterschreitung des A/D Wandlerbereichs (Prozevariable)",
                     HARDWARE & UNCORRECTABLE},

            { 0x04, "EC3 PH.PCB",   
                    "pH/ORP/pION input PCB with Conductivity/concentration firmware"
                    "|de|Kompatibilittsfehler: Platine fr pH mit Leitfhigkeit Software",
                    HARDWARE & UNCORRECTABLE}

            { 0x08, "EC4 TC.PCB",   
                    "Toroidal conductivity input PCB with 4-electrode conductivity/concentration firmware"
                    "|de|Kompatibilittsfehler: Platine fr Toroidal-Leitfhigkeit mit 4-Elekt.-Leitfhigkeit Software",
                    HARDWARE & UNCORRECTABLE}

            { 0x10, "EC6 TE.PCB",   
                    "2-electrode conductivity input PCB with 4-electrode conductivity/concentration firmware"
                    "|de|Kompatibilittsfehler: Platine fr 2-Elekt.-Leitfhigkeit mit 4-Elekt.-Leitfhigkeit Software",
                    HARDWARE & UNCORRECTABLE}

            { 0x20, "EC7 EC.PCB",   
                    "Electrode conductivity input PCB with 4-electrode conductivity/concentration firmware"
                    "|de|Kompatibilittsfehler: Platine fr Elekt.-Leitfhigkeit mit 4-Elekt.-Leitfhigkeit Software",
                    HARDWARE & UNCORRECTABLE}

            /* bit 6 POLAR is for TE only.
            ** bit 7 is not used */
         }
      }

      VARIABLE xmtr_specific_status_1
      {
         REDEFINE TYPE BIT_ENUMERATED
         {
            { 0x01, "PC3 DRTY.SN|de|PC3 BELAG",   
                    "Dirty Sensor detected"
                    "|de|Meldung: Sensor verschmutzt",
                    HARDWARE & CORRECTABLE},

            { 0x02, "PC4 GND LP|de|PC4 ERD.KRZ",   
                    "Ground loops present or shorted sensor cable"
                    "|de|Erdschleife im Mekreis oder Kabelkurzschlu",
                    HARDWARE & CORRECTABLE},

            { 0x04, "PC6 HI.LOOP|de|PC6 OUT.HI",  
                    "Current loop above upper range"
                    "|de|Ausgangsstrom HI oberhalb des oberen Bereichswerts (+0,4 mA Hysterese)",
                    PROCESS & CORRECTABLE},

            { 0x08, "PC7 LO.LOOP|de|PC7 OUT.LO",  
                    "Current loop below lower range value"
                    "|de|Ausgangsstrom LO unterhalb des unteren Bereichswerts (-0,2 mA Hysterese)",
                    PROCESS & CORRECTABLE},

            { 0x10, "PC8 HI.PV|de|PC8 PV.HI",    
                    "PV above transmitter range"
                    "|de|Prozevariable oberhalb des darstellbaren Bereichs",
                    PROCESS & CORRECTABLE},

            { 0x20, "PC9 LO.PV|de|PC9 PV.LO",    
                    "PV below transmitter range"
                    "|de|Prozevariable unterhalb des darstellbaren Bereichs",
                    PROCESS & CORRECTABLE},

            { 0x40, "PC10 HI.TEMP|de|PC10 TMP.HI", 
                    "Temperature above transmitter range"
                    "|de|Temperatur oberhalb des darstellbaren Bereichs",
                    PROCESS & CORRECTABLE},

            { 0x80, "PC11 LO.TEMP|de|PC11 TMP.LO", 
                    "Temperature below transmitter range"
                    "|de|Temperatur unterhalb des darstellbaren Bereichs",
                    PROCESS & CORRECTABLE}
         }
      }

      VARIABLE xmtr_specific_status_2
      {
         REDEFINE TYPE BIT_ENUMERATED
         {
            { 0x01, "PC12 HI.T.AD|de|PC12 TMP.FHL",
                    "Open or missing Temperature Sensor"
                    "|de|Fehler im Temperatur-Mekreis: offen oder kein Temperaturfhler angeschlossen",
                     HARDWARE & CORRECTABLE},

            { 0x02, "PC13 LO.T.AD|de|PC13 TMP.KRZ",  
                    "Shorted Temperature Sensor"
                    "|de|Kurzschlu im Temperatur-Mekreis",
                    HARDWARE & CORRECTABLE},

            { 0x04, "PC2O BAD.SEE|de|PC20 FHL.SEE",
                    "Bad SEEPROM or bad input PCB assembly"
                    "|de|Fehler Serial EEPROM oder Eingangskartenelektronik",
                    HARDWARE & CORRECTABLE},

            { 0x08, "PC21 NO.F.CAL|de|PC21 NO.W.KAL",
                    "Missing factory cal or nonfunctional SEEPROM"
                    "|de|Fehlende Werkskalibrierung oder Fehler Serial EEPROM",
                    HARDWARE & CORRECTABLE},

            { 0x10, "PC22 BLNK.EE",
                    "Blank microprocessor EEPROM"
                    "|de|Mikroprozessor EEPROM ohne Programmierung",
                     HARDWARE & CORRECTABLE},

            { 0x20, "PC25 ROM.SUM",
                    "Incorrect EPROM checksum"
                    "|de|Fehler EPROM Checksumme",
                    HARDWARE & CORRECTABLE},

            { 0x40, "PC30 R0.F.CAL|DE|PC30 R0.W.KAL",
                    "Out of range or missing factory cal for conductivity circuit range zero"
                    "|de|Wert auerhalb des Mebereichs oder fehlende Werkskalibrierung des Leitfhigkeit-Mekreises Mebereich Null",
                    HARDWARE & CORRECTABLE},

            { 0x80, "PC31 R1.F.CAL|DE|PC31 R1.W.KAL",
                    "Out of range or missing factory cal for conductivity circuit range one"
                    "|de|Wert auerhalb des Mebereichs oder fehlende Werkskalibrierung des Leitfhigkeit-Mekreises Mebereich Eins",
                    HARDWARE & CORRECTABLE}
         }
      }
      VARIABLE xmtr_specific_status_3
      {
         REDEFINE TYPE BIT_ENUMERATED
         {
            { 0x01, "PC32 R2.F.CAL|DE|PC32 R2.W.KAL",
                    "Out of range or missing factory cal for conductivity circuit range two"
                    "|de|Wert auerhalb des Mebereichs oder fehlende Werkskalibrierung des Leitfhigkeit-Mekreises Mebereich Zwei",
                    HARDWARE & CORRECTABLE},

            { 0x02, "PC33 R3.F.CAL|DE|PC33 R3.W.KAL",
                    "Out of range or missing factory cal for conductivity circuit range three"
                    "|de|Wert auerhalb des Mebereichs oder fehlende Werkskalibrierung des Leitfhigkeit-Mekreises Mebereich Drei",
                    HARDWARE & CORRECTABLE},

            { 0x04, "PC34 R4.F.CAL|DE|PC34 R4.W.KAL",
                    "Out of range or missing factory cal for conductivity circuit range four"
                    "|de|Wert auerhalb des Mebereichs oder fehlende Werkskalibrierung des Leitfhigkeit-Mekreises Mebereich Vier",
                    HARDWARE & CORRECTABLE},

            { 0x08, "PC35 G0.F.CAL|DE|PC35 G0.W.KAL",
                    "Out of range or missing factory cal for ground loop circuit range zero" /* has a different Help text for TB82TE */
                    "|de|Wert auerhalb des Mebereichs oder fehlende Werkskalibrierung des Erdschleifen- Mekreises Mebereich Null",
                    HARDWARE & CORRECTABLE},

            { 0x10, "PC36 G1.F.CAL|DE|PC36 G1.W.KAL",
                    "Out of range or missing factory cal for ground loop circuit range one"
                    "|de|Wert auerhalb des Mebereichs oder fehlende Werkskalibrierung des Erdschleifen- Mekreises Mebereich Eins",
                    HARDWARE & CORRECTABLE},

            { 0x20, "PC37 G2.F.CAL|DE|PC37 G2.W.KAL",
                    "Out of range or missing factory cal for ground loop circuit range two"
                    "|de|Wert auerhalb des Mebereichs oder fehlende Werkskalibrierung des Erdschleifen- Mekreises Mebereich Zwei",
                    HARDWARE & CORRECTABLE},

            { 0x40, "PC38 G3.F.CAL|DE|PC38 G3.W.KAL",
                    "Out of range or missing factory cal for ground loop circuit range three"
                    "|de|Wert auerhalb des Mebereichs oder fehlende Werkskalibrierung des Erdschleifen- Mekreises Mebereich Drei",
                    HARDWARE & CORRECTABLE},

            { 0x80, "PC39 G4.F.CAL|DE|PC39 G4.W.KAL",
                    "Out of range or missing factory cal for ground loop circuit range four"
                    "|de|Wert auerhalb des Mebereichs oder fehlende Werkskalibrierung des Erdschleifen- Mekreises Mebereich Vier",
                    HARDWARE & CORRECTABLE}
         }
      }
      VARIABLE xmtr_specific_status_4
      {
         REDEFINE TYPE BIT_ENUMERATED
         {

            /* bit 0 to bit 4 are not used */

            { 0x20, "PC45 BA.F.CAL|de|PC45 3K.W.KAL",
                    "Out of range or missing factory cal for 3-k ohm Balco temp.compensator"
                    "|de|Wert auerhalb des Mebereichs oder fehlende Werkskalibrierung des 3kOhm T-Fhlers",
                    HARDWARE & CORRECTABLE},

            { 0x40, "PC46 PT.F.CAL|de|PC46 PT.W.KAL",
                    "Out of range or missing factory cal for PT100 temp. compensator"
                    "|de|Wert auerhalb des Mebereichs oder fehlende Werkskalibrierung des PT100 T-Fhlers",
                    HARDWARE & CORRECTABLE},

            { 0x80, "PC47 RT.F.CAL|de|PC47 47.W.KAL",
                    "Out of range or missing factory cal for 4.75-k ohm RTD network temp. compensator"
                    "|de|Wert auerhalb des Mebereichs oder fehlende Werkskalibrierung der 4,75 kOhm Temperaturkompensation",
                    HARDWARE & CORRECTABLE}
         }
      }
      VARIABLE xmtr_specific_status_5
      {
         REDEFINE TYPE BIT_ENUMERATED
         {
            /* bit 0 PK.F.CAL is for TE only */

            { 0x02, "PC50 R0.CHKS|DE|PC50 R0.CSUM",
                    "Incorrect or missing conductivity circuit range zero checksum"
                    "|de|Falsche oder fehlende Prfsumme fr den Leitfhigkeit-Mekreis Mebereich Null",
                    HARDWARE & CORRECTABLE},

            { 0x04, "PC51 R1.CHKS|DE|PC51 R1.CSUM",
                    "Incorrect or missing conductivity circuit range one checksum"
                    "|de|Falsche oder fehlende Prfsumme fr den Leitfhigkeit-Mekreis Mebereich Eins",
                    HARDWARE & CORRECTABLE},

            { 0x08, "PC52 R2.CHKS|DE|PC52 R2.CSUM",
                    "Incorrect or missing conductivity circuit range two checksum"
                    "|de|Falsche oder fehlende Prfsumme fr den Leitfhigkeit-Mekreis Mebereich Zwei",
                    HARDWARE & CORRECTABLE},

            { 0x10,  "PC53 R3.CHKS|DE|PC53 R3.CSUM",
                    "Incorrect or missing conductivity circuit range three checksum"
                    "|de|Falsche oder fehlende Prfsumme fr den Leitfhigkeit-Mekreis Mebereich Drei",
                    HARDWARE & CORRECTABLE},

            { 0x20, "PC54 R4.CHKS|DE|PC54 R4.CSUM",
                    "Incorrect or missing conductivity circuit range four checksum"
                    "|de|Falsche oder fehlende Prfsumme fr den Leitfhigkeit-Mekreis Mebereich Vier",
                    HARDWARE & CORRECTABLE},

            { 0x40, "PC55 G0.CHKS|DE|PC55 G0.CSUM",
                    "Incorrect or missing ground loop circuit range zero checksum"
                    "|de|Falsche oder fehlende Prfsumme fr den Erdschleifen-Mekreis Mebereich Null",
                    HARDWARE & CORRECTABLE},

            { 0x80, "PC56 G1.CHKS|DE|PC56 G1.CSUM",
                    "Incorrect or missing ground loop circuit range one checksum"
                    "|de|Falsche oder fehlende Prfsumme fr den Erdschleifen-Mekreis Mebereich Eins",
                    HARDWARE & CORRECTABLE}
         }
      }
      VARIABLE xmtr_specific_status_6
      {
         REDEFINE TYPE BIT_ENUMERATED
         {
            { 0x01, "PC57 G2.CHKS|DE|PC57 G2.CSUM",
                    "Incorrect or missing ground loop circuit range two checksum"
                    "|de|Falsche oder fehlende Prfsumme fr den Erdschleifen-Mekreis Mebereich Zwei",
                    HARDWARE & CORRECTABLE},

            { 0x02, "PC58 G3.CHKS|DE|PC58 G3.CSUM",
                    "Incorrect or missing ground loop circuit range three checksum"
                    "|de|Falsche oder fehlende Prfsumme fr den Erdschleifen-Mekreis Mebereich Drei",
                    HARDWARE & CORRECTABLE},

            { 0x04, "PC59 G4.CHKS|DE|PC59 G4.CSUM",
                    "Incorrect or missing ground loop circuit range four checksum"
                    "|de|Falsche oder fehlende Prfsumme fr den Erdschleifen-Mekreis Mebereich Vier",
                    HARDWARE & CORRECTABLE}

            /* bit 3 to bit 7 are not used */
         }
      }
      VARIABLE xmtr_specific_status_7
      {
         REDEFINE TYPE BIT_ENUMERATED
         {
            { 0x01, "PC65 BA.CHKS|de|PC65 3K.CSUM",
                    "Incorrect or missing 3-k ohm Balco temp. compensator checksum"
                    "|de|Falsche oder fehlende 3kOhm T-Fhler Checksumme",
                    HARDWARE & CORRECTABLE},

            { 0x02, "PC66 PT.CHKS|de|PC66 PT.CSUM",
                    "Incorrect or missing PT100 temp. compensator checksum"
                    "|de|Falsche oder fehlende PT100 T-Fhler Checksumme",
                    HARDWARE & CORRECTABLE},

            { 0x04, "PC67 RT.CHKS|de|PC67 47.CSUM",
                    "Incorrect or missing 4.75-k ohm network temp. compensator checksum"
                    "|de|Falsche oder fehlende Prfsumme der 4,75 kOhm Temperaturkompensation",
                    HARDWARE & CORRECTABLE},

            /* bit 3 PK.CHKS is for TE */

            { 0x10, "PC70 HI.G.L.AD",
                    "Ground loop signal above transmitter A/D range"
                    "|de|berschreitung des A/D Wandler Bereichs (Erdschleifen-Signal)",
                    HARDWARE & CORRECTABLE},

            { 0x20, "PC71 LO.G.L.AD",
                   "ground loop signal below transmitter A/D range"
                    "|de|Unterschreitung des A/D Wandler Bereichs (Erdschleifen-Signal)",
                   HARDWARE & CORRECTABLE},

            { 0x40, "PC72 HI.D.S.AD",
                    "Dirty sensor signal above transmitter A/D range"
                    "|de|berschreitung des A/D Wandler Bereichs (Sensor Verschmutzungs-Signal)",
                    HARDWARE & CORRECTABLE},

            { 0x80, "PC73 LO.D.S.AD",
                   "Dirty Sensor signal below transmitter A/D range"
                    "|de|Unterschreitung des A/D Wandler Bereichs (Sensor Verschmutzungs-Signal)",
                   HARDWARE & CORRECTABLE}
         }
      }
      VARIABLE xmtr_specific_status_8
      {
         REDEFINE TYPE BIT_ENUMERATED
         {
            /* bits 0 and 1 are not used */

            { 0x04, "PV LO OFFSET|de|PV NULLP MIN",
                    "PV calibration Offset too low"
                    "|de|PV Kalibrierung Nullpunktverschiebung zu klein",
                    SELF_CORRECTING},

            { 0x08, "PV LO SLOPE|de|PV STEILH MIN",
                    "PV calibration Slope too low"
                    "|de|PV Kalibrierung Steilheit zu klein",
                    SELF_CORRECTING},

            { 0x10, "PV HI OFFSET|de|PV NULLP MAX",
                    "PV calibration Offset too high"
                    "|de|PV Kalibrierung Nullpunktverschiebung zu gro",
                    SELF_CORRECTING},

            { 0x20, "PV HI SLOPE|de|PV STEILH MAX",
                    "PV calibration Slope too high"
                    "|de|PV Kalibrierung Steilheit zu gro",
                    SELF_CORRECTING},

            { 0x40, "Temp Offset Warning|de|Temp. Offset warnung",
                    "Temperature calibration Offset unacceptable"
                    "|de|Fehler beim Kalibrieren der Temperaturmessung:Offset auerhalb Toleranz",
                    SELF_CORRECTING},

            { 0x80, "Temp Slope Warning|de|Temp. Steilheit warnung",
                    "Temperature calibration Slope unacceptable"
                    "|de|Fehler beim Kalibrieren der Temperaturmessung: Steilheit auerhalb Toleranz",
                    SELF_CORRECTING}
         }
      }

      COMMAND read_additional_transmitter_status
      {
         REDEFINE TRANSACTION
         {
            REQUEST {}

            REPLY
            {
               response_code, device_status,
               xmtr_specific_status_0,
               xmtr_specific_status_1,
               xmtr_specific_status_2,
               xmtr_specific_status_3,
               xmtr_specific_status_4,
               xmtr_specific_status_5,
               0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, /* bytes 6 to 13 are unused */ 
               xmtr_specific_status_6,
               xmtr_specific_status_7,
               xmtr_specific_status_8
            }
         }
      }
   }
}

/**************************************************************************
**		TRANSMITTER VARIABLES DEFINITION
****************************************************************************/
VARIABLE pv_units
{
    HELP [digital_units_help];

    CLASS CORRECTION;
    LABEL "PV eng. unit|de|Dimension der PV";
    TYPE ENUMERATED
    {
       { 57,  [Percent],   [percent_help] },
       { 139, [parts_per_million], [parts_per_million_help] },
       { 240,  "ppb",    "Parts per Billion" },
       { 251, " ", [none] }
    }
    HANDLING READ&WRITE;	/* This is configured as READ&WRITE, it is treated
					** as READ ONLY in the DD. The WRITE is used
					** while working with offline configurations
                              ** so that configurations in all allowable eng. units
                              ** can be created and stored. */
}

VARIABLE pv_value
{
    HELP "Concentration - Field Device Variable Digital value that changes"
         " magnitude with respect to time with no user intervention."
         "|de|Konzentration- Digitalwert des Feldgertes, der aus der Leitfhigkeit mit Hilfe im"
                  " Gert gespeicherter Konzentrations-Kurven automatisch errechnet wird.";

    CLASS DYNAMIC & CORRECTION;
    LABEL [pv];
    TYPE FLOAT
    {
         DISPLAY_FORMAT ".3f";
    }
    HANDLING READ;
}

VARIABLE sensor_upper_range_limit
{
   HELP [upper_sensor_limit_help];
   CLASS CORRECTION;
   LABEL [pv_usl];
   HANDLING READ;
   TYPE FLOAT
   {
      DISPLAY_FORMAT ".3f";
   }
}

VARIABLE sensor_lower_range_limit
{
   HELP [lower_sensor_limit_help];
   CLASS CORRECTION;
   LABEL [pv_lsl];
   HANDLING READ;
   TYPE FLOAT
   {
      DISPLAY_FORMAT ".3f";
   }
}

VARIABLE sensor_minimum_span
{
   HELP [minimum_span_help];
   CLASS CORRECTION;
   LABEL [pv_minimum_span];
   HANDLING READ;
   TYPE FLOAT
   {
      DISPLAY_FORMAT ".3f";
   }
}

VARIABLE pv_damping_value
{
   HELP [seconds_damping_value_help];
   LABEL [pv_damping_value];
   CLASS CORRECTION;
   HANDLING READ&WRITE;
   CONSTANT_UNIT [sec];
   TYPE FLOAT
   {
      DISPLAY_FORMAT ".2f";
      EDIT_FORMAT "8.2f";
      MIN_VALUE 0.00;
      MAX_VALUE 99.90;
   }
}

VARIABLE analog_value
{
   HELP [analog_value_help];
   CLASS ANALOG_OUTPUT & DYNAMIC;
   LABEL [analog_output_value];
   HANDLING READ&WRITE;
   CONSTANT_UNIT [mA];
   TYPE FLOAT
   {
      DISPLAY_FORMAT ".2f";
      EDIT_FORMAT "8.2f";
   }
}

VARIABLE fail_mode
{
   HELP [alarm_code_help];
   CLASS ANALOG_OUTPUT;
   LABEL [alarm_select];
   HANDLING READ;
   TYPE ENUMERATED
   {
      ALARM_SELECTION_CODE(0),
      ALARM_SELECTION_CODE(1)
   }
}

VARIABLE percent_range
{
   HELP [percent_of_range_help];
   CLASS INPUT & DYNAMIC;
   LABEL [pv_percent_range];
   CONSTANT_UNIT [Percent];
   HANDLING READ;
   TYPE FLOAT
   {
      DISPLAY_FORMAT ".2f";
   }
}

VARIABLE transfer_function
{
   HELP [transfer_function_help];
   CLASS INPUT;
   LABEL [transfer_function];
   HANDLING READ;
 
   TYPE ENUMERATED
   {
      { 0,  [linear],  [linear_help] }	/* Linear */
    }
}

VARIABLE pv_upper_range_value
{
   HELP [upper_range_value_help];
   LABEL [pv_urv];
   CLASS INPUT;
   HANDLING READ&WRITE;
   TYPE FLOAT
   {
      DISPLAY_FORMAT ".3f";
      EDIT_FORMAT "11.3f";
   }
}

VARIABLE pv_lower_range_value
{
   HELP [lower_range_value_help];
   LABEL [pv_lrv];
   CLASS INPUT;
   HANDLING READ&WRITE;
   TYPE FLOAT
   {
      DISPLAY_FORMAT ".3f";
      EDIT_FORMAT "11.3f";
   }
}

VARIABLE sensor_serial_number
{
   HELP [sensor_serial_number_help];
   CLASS DEVICE;
   LABEL [pv_sensor_serial_number];
   HANDLING READ&WRITE;
   TYPE UNSIGNED_INTEGER (3)
   {
      DISPLAY_FORMAT  "8u";
      EDIT_FORMAT "8u" ;
      MAX_VALUE 16777215;
   }
}

VARIABLE temp_unit
{
    CLASS CORRECTION;
    LABEL "Temp Unit|de|Temp.-Einheit";
    HANDLING READ;
    TYPE ENUMERATED
    {
       {32, [degC], [degrees_celsius_help]}
       {33, [degF], [degrees_fahrenheit_help]} 
    }
}

VARIABLE temp_value
{
   CLASS CORRECTION & DYNAMIC;
   LABEL [temperature];
   HELP  " Temperature- In Auto temp comp mode, it is the Temp read with"
         " the temp sensor. In Manual temp comp mode, it is the user entered"
         " temp value"
         "|de|In der Betriebsart \"Automatische Temp.-komp.\" wird die mit"
         " dem Temperaturfhler gemessene Temperatur angezeigt. In der"
         " Betriebsart \"Manuelle Temp.-komp.\" wird die vom Benutzer"
         " eingegebene Temperatur angezeigt";
   HANDLING READ;
   TYPE FLOAT
   {
      DISPLAY_FORMAT ".2f";
   }
}


/*************************************************************************
**		ARRAYs and COLLECTION
***************************************************************************/
ARRAY OF COLLECTION dynamic_variables
{
   ELEMENTS
   {
      PRIMARY,   pv, [pv];
      SECONDARY, sv, [temperature];
   }
}

COLLECTION OF VARIABLE pv
{
   LABEL [pv];
   HELP [primary_variable_help];
   MEMBERS
   {
      /* Sensor Correction Information */
      DIGITAL_UNITS, pv_units, [units], [digital_units_help];
      DIGITAL_VALUE, pv_value, [blank], [digital_value_help];
      SENSOR_UNITS, pv_units, [sensor_unit], [digital_units_help];
      UPPER_SENSOR_LIMIT, sensor_upper_range_limit, [usl], [upper_sensor_limit_help];
      LOWER_SENSOR_LIMIT, sensor_lower_range_limit, [lsl], [lower_sensor_limit_help];
      MINIMUM_SPAN, sensor_minimum_span, [minimum_span], [minimum_span_help];
      DAMPING_VALUE, pv_damping_value, [damping_value], [seconds_damping_value_help];

      /* Analog Output Information */
      ANALOG_VALUE, analog_value, [analog_output_value],[analog_value_help];
      ALARM_CODE, fail_mode, [alarm_select], [alarm_code_help];

      /* Input Information */
      PERCENT_RANGE, percent_range, [percent_of_range], [percent_of_range_help];
      TRANSFER_FUNCTION, transfer_function, [transfer_function],[transfer_function_help];
      RANGE_UNITS, pv_units, [range_units], [range_units_help];
      UPPER_RANGE_VALUE, pv_upper_range_value, [urv], [upper_range_value_help];
      LOWER_RANGE_VALUE, pv_lower_range_value, [lrv], [lower_range_value_help];

      /* Device Information */
      SENSOR_SERIAL_NUMBER, sensor_serial_number, [sensor_serial_number], [sensor_serial_number_help];
   }
}

COLLECTION OF VARIABLE sv
{
   LABEL [sv];
   HELP [secondary_variable_help];
   MEMBERS
   {
      DIGITAL_UNITS, temp_unit,  [units], [digital_units_help];
      DIGITAL_VALUE, temp_value, [blank], " Temperature- In Auto temp comp mode,"
                                          " it is the Temp read with the temp sensor."
                                          " In Manual temp comp mode, it is the user"
                                          " entered temp value"
         "|de|In der Betriebsart \"Automatische Temp.-komp.\" wird die mit"
         " dem Temperaturfhler gemessene Temperatur angezeigt. In der"
         " Betriebsart \"Manuelle Temp.-komp.\" wird die vom Benutzer"
         " eingegebene Temperatur angezeigt";
   }
}


ARRAY OF VARIABLE loop_warning_variables
{
   ELEMENTS
   {
      1, pv_upper_range_value;
      2, pv_lower_range_value;
      3, pv_damping_value;
   }
}

/**********************************************************************
		RELATIONS
**********************************************************************/

UNIT pv_unit_relation
{
        pv_units:
                pv_value,
                pv_upper_range_value,
                pv_lower_range_value,
                sensor_upper_range_limit,
                sensor_lower_range_limit,
                sensor_minimum_span,
                one_point_PV_value,
                one_point_PV_capture
}

UNIT temp_unit_relaton
{
        temp_unit:
                temp_value,
                temp_cal_value
}


/* This is to read back the PV ranges after writing to them */
REFRESH upper_range_values_refresh
{
   PV.UPPER_RANGE_VALUE:
            PV.UPPER_RANGE_VALUE
}
REFRESH lower_range_values_refresh
{
   PV.LOWER_RANGE_VALUE:
            PV.LOWER_RANGE_VALUE
}
/*********************************************************************
*	TRANSMITTER SPECIFIC COMMANDS AND RELATED VARIABLES
*********************************************************************/
/*
** COMMAND 128 Reset Calibration
*/

COMMAND reset_to_factory_cal
{
    NUMBER 128;
    OPERATION COMMAND;
    TRANSACTION
    {
        REQUEST {}
        REPLY
        {
            response_code, device_status
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,    [no_command_specific_errors];
        7,  MODE_ERROR, "In calibration security mode|de|Schreibschutz fr Kalibrieren ist eingeschaltet";
        32, MODE_ERROR, "Transmitter Busy - Cannot Process Command"
                        "|de|Transmitter ausgelastet-Befehl kann nicht ausgefhrt werden";
    }
}

/*
** VARIABLES for cmd 129 and 130
*/
VARIABLE PV_efficiency
{
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
        EDIT_FORMAT "10.3f";
    }
    CLASS CORRECTION;
    LABEL "PV slope|de|PV Steilheit";
    HELP  "PV sensor slope value|de|PV Sensor-Steilheit";
    HANDLING READ & WRITE;
}

VARIABLE PV_offset
{
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
        EDIT_FORMAT "10.3f";
    }
    CLASS CORRECTION;
    LABEL "PV offset";
    HELP  "PV sensor offset value.|de|PV Sensor Nullpunktverschiebung";
    HANDLING READ & WRITE;
    CONSTANT_UNIT [uSiemen_cm];
}

VARIABLE temp_slope
{
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
        EDIT_FORMAT "10.3f";
    }
    CLASS CORRECTION;
    LABEL "Temperature sensor Slope|de|Temperatursensor Steigung";
    HELP  "Temperature sensor Slope|de|Temperatursensor Steigung";
    HANDLING READ & WRITE;
}

VARIABLE temp_offset
{
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".2f";
        EDIT_FORMAT "8.2f";
    }
    CLASS CORRECTION;
    LABEL "Temperature sensor offset|de|Temperatursensor Nullpunkt";
    HELP  "Temperature sensor offset value.|de|Temperatursensor Nullpunkt";
    HANDLING READ & WRITE;
    CONSTANT_UNIT [degC];
}

/*
** COMMAND 129 Read Calibration Efficiency and Offset
*/
COMMAND read_cal_effic_and_offset
{
    NUMBER 129 ;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST {}
        REPLY
        {
            response_code, device_status,
            PV_efficiency,
            PV_offset,
            temp_slope,
            temp_offset
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,    [no_command_specific_errors];
        32, MODE_ERROR, "Transmitter Busy - Cannot Process Command"
                        "|de|Transmitter ausgelastet-Befehl kann nicht ausgefhrt werden";
    }
}


/*
** COMMAND 130 Write Calib Effic. and Offset
*/
COMMAND write_cal_effic_and_offset
{
    NUMBER 130;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            PV_efficiency,
            PV_offset,
            temp_slope,
            temp_offset
        }
        REPLY
        {
            response_code, device_status,
            PV_efficiency,
            PV_offset,
            temp_slope,
            temp_offset
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,    [no_command_specific_errors];
        5,  MISC_ERROR, [too_few_data_bytes_recieved];
        7,  MODE_ERROR, "In calibration security mode|de|Schreibschutz fr Kalibrieren ist eingeschaltet";
        32, MODE_ERROR, "Transmitter Busy - Cannot Process Command"
                        "|de|Transmitter ausgelastet-Befehl kann nicht ausgefhrt werden";
    }
}


/*
** VARIABLE for cmd 135 
*/
VARIABLE one_point_PV_capture
{
    LABEL "Conc cal value|de|Kalibrierwert fr Konzentration";
    CLASS CORRECTION;
    HANDLING READ;
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
 }

/*
** COMMAND 135 One point calib PV value capture command
*/
COMMAND one_point_capture
{
    NUMBER 135;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST{}
        REPLY
        {
            response_code, device_status,
            pv_units,
            one_point_PV_capture
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,    [no_command_specific_errors];
        32, MODE_ERROR, "Transmitter Busy - Cannot Process Command"
                        "|de|Transmitter ausgelastet-Befehl kann nicht ausgefhrt werden";
    }
}

/*
** VARIABLE for cmd 136
*/
VARIABLE one_point_PV_value
{
    LABEL "Conc cal value|de|Kalibrierwert fr Konzentration";
    HELP "Concentration calibration value|de|Kalibrierwert fr Konzentration";
    CLASS CORRECTION;
    HANDLING WRITE;
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
        EDIT_FORMAT "11.3f";
    }
 }

/*
** COMMAND 136 PV One Point Calib command
*/
COMMAND one_point_cal
{
    NUMBER 136;
    OPERATION COMMAND;
    TRANSACTION
    {
        REQUEST {
            pv_units,
            one_point_PV_value
            }
        REPLY
        {
            response_code, device_status
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,    [no_command_specific_errors];
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  MISC_ERROR, [too_few_data_bytes_recieved];
        7,  MODE_ERROR, "In calibration security mode|de|Schreibschutz fr Kalibrieren ist eingeschaltet";
        12, MISC_ERROR, [invalid_selection]; /* invalid eng units*/
        32, MODE_ERROR, "Transmitter Busy - Cannot Process Command"
                        "|de|Transmitter ausgelastet-Befehl kann nicht ausgefhrt werden";
    }
}


/*
** VARIABLE for cmd 138
*/
VARIABLE temp_cal_value
{
    LABEL "Temp. calibration value|de|Temp. Kalibrierwert";
    HELP  "Temperature calibration value|de|Temperatur Kalibrierwert";
    CLASS CORRECTION;
    HANDLING WRITE;
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".2f";
        EDIT_FORMAT "8.2f";
    }
}


/*
** COMMAND 138 Temp Calib command
*/
COMMAND temp_sensor_cal
{
    NUMBER 138;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            temp_unit,
            temp_cal_value
        }
        REPLY
        {
            response_code, device_status
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,    [no_command_specific_errors];
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  MISC_ERROR, [too_few_data_bytes_recieved];
        7,  MODE_ERROR, "In calibration security mode|de|Schreibschutz fr Kalibrieren ist eingeschaltet";
        12, MISC_ERROR, [invalid_selection]; /* invalid eng units*/
        32, MODE_ERROR, "Transmitter Busy - Cannot Process Command"
                        "|de|Transmitter ausgelastet-Befehl kann nicht ausgefhrt werden";
    }
}

/********************************************************************
			MENUS
**********************************************************************/
MENU root_menu
{
    LABEL [bus_devices];
    ITEMS
    {
        xmtr_setup,					/* Common menu */
        pv_value (DISPLAY_VALUE, READ_ONLY),	/* variable */
        analog_value  (DISPLAY_VALUE, READ_ONLY),	/* variable */
        PV.LOWER_RANGE_VALUE (DISPLAY_VALUE),		/* variable */
        PV.UPPER_RANGE_VALUE (DISPLAY_VALUE)		/* variable */
    }
}


MENU xmtr_setup
{
    LABEL [device_setup];
    ITEMS
    {
        view_pv,		/* Common menu */
        diag_and_service,	/* Common menu */
        basic_xmtr_setup,	/* Common menu */
        detailed_xmtr_setup,	/* Common menu */
        parameter_review (REVIEW),	/* xmtr variable review list */
        refresh_variables	/* method */
    }
}


MENU view_pv
{
    LABEL [process_variables];
    ITEMS
    {	/* all variables */
        pv_value (DISPLAY_VALUE, READ_ONLY),
        analog_value  (DISPLAY_VALUE, READ_ONLY),
        PV.PERCENT_RANGE (DISPLAY_VALUE, READ_ONLY),
        temp_value (DISPLAY_VALUE, READ_ONLY)
    }
}


MENU diag_and_service
{
    LABEL [diagnostics_and_service];
    ITEMS
    {
        xmtr_calibration,	/* menu */
        loop_test,		/* common method */
        status,			/* menu */
        xmtr_reset		/* xmtr specific method */
    }
}


MENU xmtr_calibration
{
    LABEL [calibration];
    ITEMS
    {
        conc_cal_method,        /* xmtr_specific_method */
        temp_cal_method,	/* xmtr specific method */
        edit_cal_method,	/* xmtr specific method */
        reset_cal_method,	/* xmtr specific method */
        dac_trim,		/* common method */
        scaled_dac_trim		/* common method */
    }
}


MENU status
{
    LABEL [status];
    ITEMS
    {   /* xmtr specofic variables */
        xmtr_specific_status_0 (DISPLAY_VALUE),
        xmtr_specific_status_1 (DISPLAY_VALUE),
        xmtr_specific_status_2 (DISPLAY_VALUE),
        xmtr_specific_status_3 (DISPLAY_VALUE),
        xmtr_specific_status_4 (DISPLAY_VALUE),
        xmtr_specific_status_5 (DISPLAY_VALUE),
        xmtr_specific_status_6 (DISPLAY_VALUE)
        xmtr_specific_status_7 (DISPLAY_VALUE)
        xmtr_specific_status_8 (DISPLAY_VALUE)
    }
}

MENU basic_xmtr_setup
{
    LABEL [basic_setup];
    ITEMS
    {
        tag (DISPLAY_VALUE),	/* variable */
        PV.DIGITAL_UNITS (DISPLAY_VALUE, READ_ONLY),  /* variable */
        range_setup,		/* edit display */
        PV.DAMPING_VALUE       /* variable */
    }
}


MENU detailed_xmtr_setup
{
   LABEL [detailed_setup];
   ITEMS
   {  /* all menus */
      sensor_menu,
      signal_conditioning_menu,
      analog_output_menu,
      HART_parameters,
      device_info_menu
   }
}

MENU sensor_menu
{
    LABEL [sensors];
    ITEMS
    {
        pv_value (DISPLAY_VALUE),	/* variable */
        PV.DIGITAL_UNITS (DISPLAY_VALUE, READ_ONLY),  /* variable */
        sensor_serial_number,		/* variable */
        sensor_info_menu (REVIEW)	/* menu */
    }
}

MENU sensor_info_menu
{
   LABEL [sensor_info];
   ITEMS
   {  /* all variables */
      sensor_lower_range_limit (DISPLAY_VALUE, READ_ONLY),
      sensor_upper_range_limit (DISPLAY_VALUE, READ_ONLY),
      sensor_minimum_span (DISPLAY_VALUE, READ_ONLY),
      PV_efficiency (DISPLAY_VALUE, READ_ONLY),
      PV_offset (DISPLAY_VALUE, READ_ONLY),
      temp_slope (DISPLAY_VALUE, READ_ONLY),
      temp_offset (DISPLAY_VALUE, READ_ONLY)
   }
}

MENU signal_conditioning_menu
{
   LABEL [signal_condition];
   ITEMS
   {  /* all variables */
      PV.LOWER_RANGE_VALUE (DISPLAY_VALUE),
      PV.UPPER_RANGE_VALUE (DISPLAY_VALUE),
      PV.DAMPING_VALUE (DISPLAY_VALUE),
      PV.TRANSFER_FUNCTION (DISPLAY_VALUE),
      PV.PERCENT_RANGE (DISPLAY_VALUE)
   }
}

MENU analog_output_menu
{
   LABEL [analog_output];
   ITEMS
   {
      analog_value (DISPLAY_VALUE,READ_ONLY),	/* variable */
      fail_mode (DISPLAY_VALUE),			/* variable */
      loop_test,				/* common method */
      dac_trim,					/* common method */
      scaled_dac_trim				/* common method  */
   }
}

MENU HART_parameters
{
   LABEL "HART Parameters|de|HART Parameter";
   ITEMS
   {  /* variables */ 
      polling_address (DISPLAY_VALUE),
      response_preambles (DISPLAY_VALUE),
      request_preambles (DISPLAY_VALUE)
   }
}


MENU device_info_menu
{
    LABEL [device_information];
    ITEMS
    {  /* variables + 1 menu */
       manufacturer_id (DISPLAY_VALUE),
       device_type (DISPLAY_VALUE),
       device_id (DISPLAY_VALUE),
       tag (DISPLAY_VALUE),
       date (DISPLAY_VALUE),
       write_protect (DISPLAY_VALUE),
       descriptor,
       message,
       final_assembly_number,
       device_revisions_menu			/* menu */
    }
}

MENU device_revisions_menu
{
    LABEL [revision_numbers];
    ITEMS
    {	/* variables */
        universal_revision (DISPLAY_VALUE),
        transmitter_revision (DISPLAY_VALUE),
        software_revision (DISPLAY_VALUE)
    }
}


MENU parameter_review
{
    LABEL [review];
    ITEMS
    {	/* all variables */
        tag (DISPLAY_VALUE, READ_ONLY),
        polling_address (DISPLAY_VALUE, READ_ONLY),
        PV.LOWER_RANGE_VALUE (DISPLAY_VALUE, READ_ONLY),
        PV.UPPER_RANGE_VALUE (DISPLAY_VALUE, READ_ONLY),
        PV.LOWER_SENSOR_LIMIT (DISPLAY_VALUE, READ_ONLY),
        PV.UPPER_SENSOR_LIMIT (DISPLAY_VALUE, READ_ONLY),
        PV.MINIMUM_SPAN (DISPLAY_VALUE, READ_ONLY),
        pv_damping_value (DISPLAY_VALUE, READ_ONLY),
        fail_mode (DISPLAY_VALUE, READ_ONLY),
        write_protect (DISPLAY_VALUE, READ_ONLY),
        final_assembly_number (DISPLAY_VALUE, READ_ONLY),
        PV.SENSOR_SERIAL_NUMBER (DISPLAY_VALUE, READ_ONLY),
        descriptor (DISPLAY_VALUE, READ_ONLY),
        message (DISPLAY_VALUE, READ_ONLY),
        date (DISPLAY_VALUE, READ_ONLY),
        manufacturer_id (DISPLAY_VALUE, READ_ONLY),
        private_label_distributor (DISPLAY_VALUE, READ_ONLY),
        device_type (DISPLAY_VALUE, READ_ONLY),
        universal_revision (DISPLAY_VALUE, READ_ONLY),
        transmitter_revision (DISPLAY_VALUE, READ_ONLY),
        software_revision (DISPLAY_VALUE, READ_ONLY),
        hardware_revision (DISPLAY_VALUE, READ_ONLY),
        request_preambles (DISPLAY_VALUE, READ_ONLY),
        device_id (DISPLAY_VALUE, READ_ONLY)
    }
}

MENU hot_key
{
    LABEL [hot_key];
    ITEMS
    {
        range_setup,	/* edit display */
        xmtr_calibration	/* menu */
    }
}


MENU upload_variables
{
    LABEL "Offline Configuration Parameters";
    ITEMS
    {
        tag,
        polling_address,
        pv_units,
        pv_lower_range_value,
        pv_upper_range_value,
        pv_damping_value,
        message,
        descriptor,
        date,
        final_assembly_number,
        sensor_serial_number
    }
}

/*********************************************************************
			EDIT DISPLAYS
*********************************************************************/
WRITE_AS_ONE range_update_wao
{
    PV.LOWER_RANGE_VALUE
    PV.UPPER_RANGE_VALUE
}


EDIT_DISPLAY range_setup
{
    LABEL "PV Range Setup|de|PV-Bereich Festlegung";
    EDIT_ITEMS
    {
        range_update_wao
    }
    DISPLAY_ITEMS
    {
        PV.DIGITAL_UNITS,
        PV.UPPER_SENSOR_LIMIT,
        PV.LOWER_SENSOR_LIMIT,
        PV.MINIMUM_SPAN
    }
}

/*********************************************************************
			METHODS
*********************************************************************/

/* The Transmitter Reset function forces the transmitter to issue a 
** hardware reset.
*/

METHOD xmtr_reset
{
   LABEL "Reset Transmitter|de|Reset Meumformer";
   HELP  "Reset Transmitter - Instructs the transmitter to generate a reset. "
         "The result is similar to the effects of briefly interrupting power to the transmitter. "
         "This can be used to recover from an error condition if needed."
         "|de|Reset Meumformer- Anweisung an den Meumformer, einen Reset"
         " zu erzeugen. Die Wirkung ist die gleiche, wie eine kurze"
         " Unterbrechung der Hilfsenergie. Kann benutzt werden, um aus"
         " einem Fehlerzustand wieder in den Normalbetrieb zu gelangen";
   CLASS DEVICE;

   DEFINITION
   {
      char disp_string1[60];
      char status[STATUS_SIZE];
      int slen1;

      slen1 =60;

      XMTR_IGNORE_ALL_RESPONSE_CODES();
      IGNORE_ALL_DEVICE_STATUS();
      XMTR_IGNORE_ALL_DEVICE_STATUS();
      XMTR_RETRY_ON_NO_DEVICE();

      get_dictionary_string(remove_loop_from_auto,disp_string1,slen1);
      ACKNOWLEDGE(disp_string1);
      add_abort_method(return_to_normal);
      ACKNOWLEDGE("Transmitter will be Reset! Continue?|de|Meumformer"
                  " wird gleich zurckgesetzt! Fortfahren?");

	/* Cannot use send() to send cmd 42 because it will transmit 
      ** a cmd 48 if the MORE status bit is set. This cmd 48 will not
      ** be responded by the xmtr since it is already reset, and will 
      ** cause an error. But the disadvantage with using send_command()
      ** is that we will not get the cmd 42 response code back and 
      ** hence we will not be able to display the response code. */
      send_command(42);

      DELAY(7,"Transmitter Reset is in process, please wait.|de|Meumformer"
              " wird zurckgesetzt, bitte warten.");
      PUT_MESSAGE("Reading current values of all transmitter variables,"
                  " please wait.|de|Lese aktuelle Werte aler Meumformer"
                  " - Variablen, bitte warten");
      SET_NUMBER_OF_RETRIES(10);
      RETRY_ON_ALL_RESPONSE_CODES();

      send_command(12);
      send_command(13);
      send_command(15);
      send_command(16);
      send_command(129);

      save_values();

      DELAY(2,"Transmitter Reset successful|de|Meumformer - Reset"
              " erfolgreich");
      process_abort();
   }
}


/*
**		Concentration ONE Point Cal Method
*/
METHOD conc_cal_method
{
   LABEL "Concentration cal|de|Konzentration Kal.";
   CLASS CORRECTION;

   HELP "Calibration of process sensor input via one-point smart"
        " calibration that adjusts offset and slope."
        "|de|Kalibrierung des Prozesensor-Eingangs mit intelligenter Einpunkt-Kalibrierung,"
                " die Nullpunktverschiebung und Steilheit abgleicht";

   DEFINITION
   {
      char status[STATUS_SIZE];
      char xmtr_status[STATUS_SIZE];
      long varid[4];
      char disp_string1[60];
      int slen1, retry;
      char xmtr_data[MAX_XMTR_STATUS_LEN];

      slen1=60;

      varid[0] = VARID(PV.DIGITAL_VALUE);
      varid[1] =VARID(PV_efficiency);
      varid[2] =VARID(PV_offset);
      varid[3] = VARID(PV.DIGITAL_UNITS);

      /*
      **  1. Display warnings
      */
      get_dictionary_string(remove_loop_from_auto,disp_string1,slen1);
      ACKNOWLEDGE(disp_string1);
      add_abort_method(return_to_normal);

      get_dictionary_string(this_will_affect_snsr_cal,disp_string1,slen1);
      ACKNOWLEDGE(disp_string1);

      IGNORE_ALL_RESPONSE_CODES();
      XMTR_IGNORE_ALL_RESPONSE_CODES();
      IGNORE_ALL_DEVICE_STATUS();
      XMTR_IGNORE_ALL_DEVICE_STATUS();

      RETRY_ON_RESPONSE_CODE(32);
      XMTR_RETRY_ON_RESPONSE_CODE(32);

      /*
      **  2. Loop for One point cal
      */
      do
      {
         /*
         **  3. Display Live PV and let user hit OK when PV is stable
         **     Then send command 135 to capture the one point cal value.
         */
         acknowledge("Press OK when PV is stable\n %0 %{3}"
                     "|de|OK drcken, wenn Prozewert stabil ist\n %0 %{3}",varid);
         send(135,status);

         vassign(one_point_PV_value, one_point_PV_capture);

         /*
         **  4. Display the captured PV value and let the user modify it.
         **     Then send command 136 with the new value.
         **
         **     Send command 48 if it has not already been sent in ext_send_command() 
         **
         */
         GET_DEV_VAR_VALUE("Enter new value:|de|Neuen Wert eingeben:", one_point_PV_value);

         /* ext_send_command sends cmd 48 if more status is indicated 
         ** as a response to cmd 136 */
         ext_send_command(136, status, xmtr_status, xmtr_data);

         if(status[STATUS_RESPONSE_CODE])
         {
            display_response_status(136, status[STATUS_RESPONSE_CODE]);
            get_dictionary_string(abort_last_error, disp_string1, slen1);
            DELAY(2,disp_string1);
            process_abort();
         }
         /* the below checking is required to make sure whether a cmd 48 was 
         ** issued after cmd 136. It has been issued if the bit in position 0x10 
         ** of device status (ie. more status bit) is set. Else, it has not been 
         ** issued, and should be issued by using get_more_status() */ 
         if( !(status[STATUS_DEVICE_STATUS] & 0x10) )
         {
            get_more_status(xmtr_status, xmtr_data); 
         }

         /*
         **  5. Read and display PV efficiency and offset values
         */
         send(129, status);  /* Read effic. and offset */

         acknowledge("PV Slope: %{1}\n"
                     "PV Offset: %{2} uS/cm"
                     "|de|PV Steilheit: %{1}\n"
                     "PV Offset: %{2} uS/cm", varid);
         /*
         **  6. Display any cal errors by reading it out of command 48 response
         **     If there are no cal errors, exit from the loop.
         **     If there are cal errors, let the user select whether to do the
         **     cal again.
         */
         if( xmtr_data[16] & 0x3C )
         {
            if(xmtr_data[16] & 0x04)
            {
               ACKNOWLEDGE("PV LO OFFSET|de|PV NULLP MIN");
            }
            if(xmtr_data[16] & 0x08)
            {
               ACKNOWLEDGE("PV LO SLOPE|de|PV STEILH MIN");
            }
            if(xmtr_data[16] & 0x10)
            {
               ACKNOWLEDGE("PV HI OFFSET|de|PV NULLP MAX");
            }
            if(xmtr_data[16] & 0x20)
            {
               ACKNOWLEDGE("PV HI SLOPE|de|PV STEILH MAX");
            }
            get_dictionary_string(yes_no,disp_string1,slen1);
            retry = SELECT_FROM_LIST("Calibration was not successful. Do you want to retry?|de|Die Kalibrierung war nicht erfolgreich."
                                     " Wiederholen?",disp_string1);
         }
         else
         {
            retry = 1;
            DELAY(2,"Calibration was successful|de|Die Kalibrierung war erfolgreich.");
         }
      }
      while(!retry);

      process_abort();
   }
}


/*
**		Temp Cal Method
*/
METHOD temp_cal_method
{
   LABEL "Temp. Cal|de|Temp. Kalib.";
   HELP  "Calibration of Temperature sensor input via one-point smart"
         " calibration that adjusts offset and slope"
         "|de|Kalibrierung des Temperatursensor-Eingangs mit intelligenter Einpunkt-Kalibrierung,"
                  " die Nullpunktverschiebung und Steilheit abgleicht";
   CLASS CORRECTION;

   DEFINITION
   {
      char status[STATUS_SIZE];
      char xmtr_status[STATUS_SIZE];
      char disp_string1[60];
      int slen1;
      char xmtr_data[MAX_XMTR_STATUS_LEN];

      slen1=60;

      /*
      **  1. Display warnings
      */
      get_dictionary_string(remove_loop_from_auto,disp_string1,slen1);
      ACKNOWLEDGE(disp_string1);
      add_abort_method(return_to_normal);

      IGNORE_ALL_RESPONSE_CODES();

      /*
      **  2. Read present temperature value and allow user to modify it
      */
      send_command(3);
      vassign(temp_cal_value, temp_value);
      GET_DEV_VAR_VALUE("Enter Temperature:|de|Bitte Temperatur eingeben:", temp_cal_value);

      /*
      **  3. Send new temp value with cmd 138, read cal errors with cmd 48
      */

      XMTR_RETRY_ON_RESPONSE_CODE(32);

      /* ext_send_command sends cmd 48 if more status is indicated 
      ** in cmd 138 response */
      ext_send_command(138, status, xmtr_status, xmtr_data);

      if(status[STATUS_RESPONSE_CODE])
      {
         display_response_status(138, status[STATUS_RESPONSE_CODE]);
         get_dictionary_string(abort_last_error, disp_string1, slen1);
         DELAY(2,disp_string1);
         process_abort();
      }
      /* the below checking is required to make sure whether a cmd 48 was 
      ** issued after cmd 138. It has been issued if the bit in position 0x10 
      ** of device status (ie. more status bit) is set. Else, it has not been 
      ** issued, and should be issued by using get_more_status() */ 
      if( !(status[STATUS_DEVICE_STATUS] & 0x10) )
      {
         get_more_status(xmtr_status, xmtr_data); 
      }

      /*
      **  4. Display any cal errors
      */
      if(xmtr_data[16] & 0x40)
      {
         ACKNOWLEDGE("TEMP OFFSET WARNING|de|TEMP. OFFSET WARNUNG");
      }
      if(xmtr_data[16] & 0x80)
      {
         ACKNOWLEDGE("TEMP SLOPE WARNING|de|TEMP. STEILHEIT WARNUNG");
      }
      send(129, status);
      process_abort();
   }
}


/*
**		Edit Cal Method
*/
METHOD edit_cal_method
{
   LABEL "Edit Cal|de|Bearbeiten Kalib.";
   HELP  "Manual adjustment of process sensor and temperature"
         " offset and slope values"
         "|de|Bearbeiten der Kalibrierdaten- Manuelle nderung von"
         " Steilheit (Wirkungsgrad) und Nullpunktverschiebung des"
         " Prozesensors und des Temperatursensors";

   CLASS CORRECTION;

   DEFINITION
   {
      char status[STATUS_SIZE];
      char xmtr_status[STATUS_SIZE];
      char disp_string1[60];
      int slen1;
      char xmtr_data[MAX_XMTR_STATUS_LEN];

      slen1=60;

      XMTR_IGNORE_ALL_RESPONSE_CODES();
      XMTR_IGNORE_ALL_DEVICE_STATUS();
      IGNORE_ALL_RESPONSE_CODES();
      IGNORE_ALL_DEVICE_STATUS();

      /*
      **  1. Display warnings
      */
      get_dictionary_string(remove_loop_from_auto,disp_string1,slen1);
      ACKNOWLEDGE(disp_string1);
      add_abort_method(return_to_normal);

      /*
      **  2. Read PV and temp offset and slope with cmd 129 and display the 
      **     values. Let user modify them as necessary.
      */
      send(129, status);

      GET_DEV_VAR_VALUE("Edit PV Slope|de|PV Steilheit bearbeiten", PV_efficiency);
      GET_DEV_VAR_VALUE("Edit PV Offset|de|PV Nullpunktverschiebung bearbeiten", PV_offset);
      GET_DEV_VAR_VALUE("Edit Temp. Slope|de|Temp. Steilheit bearbeiten", temp_slope);
      GET_DEV_VAR_VALUE("Edit Temp. Offset|de|Temp. Nullpunktverschiebung bearbeiten", temp_offset);

      XMTR_RETRY_ON_RESPONSE_CODE(32);
      /*
      **  3. Send the cal values with cmd 130. Read status with cmd 48.
      **     Display any errors.
      */
      /* ext_send_command sends cmd 48 if more status is indicated 
      ** as a response to cmd 130 */
      ext_send_command(130, status, xmtr_status, xmtr_data);

      if(status[STATUS_RESPONSE_CODE])
      {
         display_response_status(130, status[STATUS_RESPONSE_CODE]);
         get_dictionary_string(abort_last_error, disp_string1, slen1);
         DELAY(2,disp_string1);
         process_abort();
      }
      /* the below checking is required to make sure whether a cmd 48 was 
      ** issued after cmd 130. It has been issued if the bit in position 0x10 
      ** of device status (ie. more status bit) is set. Else, it has not been 
      ** issued, and should be issued by using get_more_status() */ 
      if( !(status[STATUS_DEVICE_STATUS] & 0x10) )
      {
         get_more_status(xmtr_status, xmtr_data); 
      }

      if(xmtr_data[16] & 0x04)
      {
         ACKNOWLEDGE("PV LO OFFSET|de|PV NULLP MIN");
      }
      if(xmtr_data[16] & 0x08)
      {
         ACKNOWLEDGE("PV LO SLOPE|de|PV STEILH MIN");
      }
      if(xmtr_data[16] & 0x10)
      {
         ACKNOWLEDGE("PV HI OFFSET|de|PV NULLP MAX");
      }
      if(xmtr_data[16] & 0x20)
      {
         ACKNOWLEDGE("PV HI SLOPE|de|PV STEILH MAX");
      }
      if(xmtr_data[16] & 0x40)
      {
         ACKNOWLEDGE("TEMP OFFSET WARNING|de|TEMP. OFFSET WARNUNG");
      }
      if(xmtr_data[16] & 0x80)
      {
         ACKNOWLEDGE("TEMP SLOPE WARNING|de|TEMP. STEILHEIT WARNUNG");
      }
      /*
      **  4. Finally read PV and temp offset and slope with cmd 129.
      */
      send(129, status);
      process_abort();
   }
}

/*
**		Reset Cal Method
*/
METHOD reset_cal_method
{
   LABEL "Reset Cal|de|Reset der Kalib.";
   HELP  "Reset the process and temperature sensor calibration data to the values set at the factory"
         "|de|Zurcksetzen der Kalibrierdaten fr Proze-und Temperaturfhler auf die werksseitig eingestellten Werte";
   CLASS CORRECTION;

   DEFINITION
   {
      char status[STATUS_SIZE];
      char disp_string1[60];
      int slen1;
      slen1=60;

      XMTR_IGNORE_ALL_RESPONSE_CODES();
      XMTR_IGNORE_ALL_DEVICE_STATUS();
      IGNORE_ALL_RESPONSE_CODES();
      IGNORE_ALL_DEVICE_STATUS();

      get_dictionary_string(remove_loop_from_auto,disp_string1,slen1);
      ACKNOWLEDGE(disp_string1);
      add_abort_method(return_to_normal);

      ACKNOWLEDGE("Press OK to Reset Calibration"
                  "|de|Bitte OK drcken, um die Kalibrierung zurckzusetzen");
      send(128,status);		/* Reset cal command */

      if(status[STATUS_RESPONSE_CODE])
      {
         display_response_status(128, status[STATUS_RESPONSE_CODE]);
         get_dictionary_string(abort_last_error,disp_string1,slen1);
         DELAY(2,disp_string1);
         process_abort();
      }

      send(129, status);	/* read cal command */

      process_abort();
    }
}
 
METHOD refresh_variables
{
   LABEL "Refresh variables|de|Variablen aktualis";
   HELP  "Read transmitter variables to refresh their values."
         " This will update any variable that was modified via the device"
         " keypad."
         "|de|Lesen der Meumformer-Variablen zum Aktualisieren der Werte."
         " Es wird jede Variable aktualisiert, die ber das Tastenfeld"
         " des Gertes verndert wurde.";
   CLASS DEVICE;

   DEFINITION
   {
      IGNORE_ALL_RESPONSE_CODES();
      XMTR_IGNORE_ALL_RESPONSE_CODES();
      IGNORE_ALL_DEVICE_STATUS();
      XMTR_IGNORE_ALL_DEVICE_STATUS();

      PUT_MESSAGE("Reading transmitter variables, please wait."
                  "|de|Lese Meumformer-Variablen, bitte warten.");

      send_command(12);
      send_command(13);
      send_command(14);
      send_command(15);
      send_command(16);
      send_command(129);

      save_values();

      process_abort();
   }
}

/****************************************************************************
**		END OF TB82EC-CONC DEVICE DESCRIPTION
****************************************************************************/
